<?php
/**
 * App api
 *
 * @since      1.0.0
 * @author     Beeketing
 *
 */

namespace BeeketingWoocommerce\Api;


use BeeketingWoocommerce\Data\Constant;
use BeeketingWordpressCommon\Api\CommonApi;
use BeeketingWordpressCommon\Data\AppSettingKeys;
use BeeketingWordpressCommon\Data\Setting;
use BeeketingWordpressCommon\Libraries\SettingHelper;

class App extends CommonApi
{
    /**
     * App constructor.
     *
     * @param $api_key
     */
    public function __construct( $api_key )
    {
        $setting_helper = SettingHelper::get_instance();
        $setting_helper->set_app_setting_key( Constant::APP_SETTING_KEY );
        $setting_helper->set_plugin_version( BEEKETINGWOOCOMMERCE_VERSION );

        parent::__construct(
            $setting_helper,
            BEEKETINGWOOCOMMERCE_PATH,
            BEEKETINGWOOCOMMERCE_API,
            $api_key,
            Constant::APP_CODE,
            Constant::PLUGIN_ADMIN_URL
        );
    }

    /**
     * Register shop
     *
     * @return bool
     */
    public function register_shop()
    {
        // If not email
        if ( !isset( $_POST['api_key'] ) ) {
            return false;
        }

        $api_key = $_POST['api_key'];
        $this->set_api_key( $api_key );
        if ( $api_key ) {
            $result = $this->update_shop_info();
            $shopResult = $this->get( 'shops/api/' . $api_key );

            // Install app success
            if ( $result && isset( $shopResult['shop']['id'] ) ) {
                // Migrate data
                if (
                    isset( $shopResult['shop']['platform'] ) &&
                    $shopResult['shop']['platform'] != \BeeketingWordpressCommon\Data\Constant::PLATFORM
                ) {
                    $this->do_migrate( $api_key, false );
                }

                $this->setting_helper->update_settings( Setting::SETTING_API_KEY, $api_key );
                $this->setting_helper->update_settings( Setting::SETTING_SHOP_ID, $shopResult['shop']['id'] );
                return $api_key;
            } else {
                $this->setting_helper->delete_settings();
            }
        }

        return false;
    }

    /**
     * Get beeketing apps
     *
     * @return array
     */
    public function get_apps()
    {
        $result = $this->get( 'wordpress/get_apps_v2' );

        if ( isset( $result['apps'] ) ) {
            return $result['apps'];
        }

        return array();
    }

    /**
     * Get oauth sign in urls
     *
     * @return array
     */
    public function oauth_sign_in_urls()
    {
        $urls = array();
        $setting_keys = SettingHelper::setting_keys();
        foreach ( $setting_keys as $app_code => $setting_key ) {
            $this->set_app_code( $app_code );
            $this->setting_helper->set_app_setting_key( $setting_key );
            $urls[$app_code] = $this->get_oauth_sign_in_url();
        }

        return $urls;
    }

    /**
     * Convert query string
     *
     * @param $key
     * @return string
     */
    public function convert_string( $key )
    {
        return '"' . $key . '"';
    }

    /**
     * Check migrate after update
     */
    public function check_migrate_after_update()
    {
        $api_key = get_option( 'beeketing_api_key' );

        // If detect old setting
        if ( $api_key ) {
            $this->do_migrate( $api_key );
        }
    }

    /**
     * Do migrate from old platform
     *
     * @param $api_key
     * @param bool $redirect
     * @return bool
     */
    private function do_migrate( $api_key, $redirect = true )
    {
        // Clean old settings.
        global $wpdb;

        // Get ignore app keys
        $app_keys_array = AppSettingKeys::get_constants();
        $app_keys_array[] = Constant::APP_SETTING_KEY;
        $app_keys_string = array_map( array( $this, 'convert_string' ), $app_keys_array );
        $app_keys = implode(', ', $app_keys_string );

        // Delete old setting
        $wpdb->query(
            $wpdb->prepare(
                'DELETE FROM ' . $wpdb->options . ' WHERE option_name LIKE %s AND option_name NOT IN (' . $app_keys . ')',
                '%beeketing_%'
            )
        );

        // Update beeketing products meta
        $wpdb->query(
            $wpdb->prepare(
                'UPDATE ' . $wpdb->postmeta . ' SET meta_key = %s WHERE meta_key = %s',
                '_beeketing_option1',
                '_bk_option1'
            )
        );

        // Update shop
        $this->set_api_key( $api_key );
        $this->setting_helper->update_settings( Setting::SETTING_API_KEY, $api_key );
        $updateResult = $this->update_shop_info( array(
            'platform' => \BeeketingWordpressCommon\Data\Constant::PLATFORM,
        ) );

        // Update shop id
        $shopResult = $this->get( 'shops/api/' . $api_key );
        $shopIdResult = false;
        if ( isset( $shopResult['shop']['id'] ) ) {
            $this->setting_helper->update_settings( Setting::SETTING_SHOP_ID, $shopResult['shop']['id'] );
            $shopIdResult = true;
        }

        // Re-install app
        $beeketing_apps_data = $this->get_apps();
        $installResult = true;
        foreach ( $beeketing_apps_data as $beeketing_app_data ) {
            if ( isset( $beeketing_app_data['is_installed'], $beeketing_app_data['app_code'] ) && $beeketing_app_data['is_installed'] ) {
                $app_code = $beeketing_app_data['app_code'];
                $this->setting_helper->switch_settings( $app_code );
                $this->set_app_code( $app_code );

                // Get app action result
                $installAppResult = $this->install_app();
                if ( !$installAppResult ) {
                    $installResult = false;
                    break;
                }
            }
        }

        // If migrate failed
        if ( !$updateResult || !$shopIdResult || !$beeketing_apps_data || !$installResult ) {
            $wpdb->query(
                $wpdb->prepare(
                    'INSERT INTO ' . $wpdb->options . ' (option_name, option_value) VALUES (%s, %s)',
                    'beeketing_api_key',
                    $api_key
                )
            );
            delete_option( Constant::APP_SETTING_KEY );

            if ( $redirect ) {
                // Redirect to plugin page
                exit(wp_redirect(admin_url()));
            } else {
                return false;
            }
        }

        // Trigger re-sync shop
        $this->post( 'wordpress/sync_shop' );

        if ( $redirect ) {
            // Redirect to plugin page
            exit(wp_redirect(admin_url('admin.php?page=' . Constant::PLUGIN_ADMIN_URL)));
        } else {
            return true;
        }
    }

    /**
     * Get cross sell app
     *
     * @return bool|mixed
     */
    public function get_cross_sell_app()
    {
        $cross_sell_apps = \BeeketingWoocommerce\Data\CrossSellApp::$APPS;
        $shop_installed_apps = $this->setting_helper->get_settings( Setting::SETTING_SHOP_INSTALLED_APPS );
        if ( $shop_installed_apps && !array_diff( $cross_sell_apps, $shop_installed_apps ) ) {
            return false;
        }

        $data_cookie_key = \BeeketingWordpressCommon\Data\Constant::COOKIE_BEEKETING_APPS_DATA;
        $beeketing_apps_data = isset( $_COOKIE[$data_cookie_key] ) && $_COOKIE[$data_cookie_key] ?
            unserialize( stripslashes( $_COOKIE[$data_cookie_key] ) ) : $this->get_apps();
        $shop_installed_apps = $new_apps_data  = array();
        if ( $beeketing_apps_data ) {
            foreach ( $beeketing_apps_data as $key => $beeketing_app_data ) {
                // Get shop installed apps
                if (
                    isset( $beeketing_app_data['is_existed'], $beeketing_app_data['app_code'] ) &&
                    $beeketing_app_data['is_existed']
                ) {
                    $shop_installed_apps[] = $beeketing_app_data['app_code'];
                }

                // Get new data for cookie
                $new_apps_data[] = array(
                    'app_code' => $beeketing_app_data['app_code'],
                    'is_existed' => $beeketing_app_data['is_existed'],
                );
            }

            // Store shop installed apps
            $this->setting_helper->update_settings( Setting::SETTING_SHOP_INSTALLED_APPS, $shop_installed_apps );

            // Set cookie
            $cookie_lifetime = \BeeketingWordpressCommon\Data\Constant::COOKIE_BEEKETING_APPS_DATA_LIFETIME;
            setcookie( $data_cookie_key, serialize( $new_apps_data ), time() + $cookie_lifetime, '/' );
        }

        $shop_installed_apps = $this->setting_helper->get_settings( Setting::SETTING_SHOP_INSTALLED_APPS );
        if ( $shop_installed_apps ) {
            $current_cross_sell_app_key = 'beeketing_current_cross_sell_app';
            $next_cross_sell_app_key = 'beeketing_next_cross_sell_app';
            $cross_sell_apps = array_diff( $cross_sell_apps, $shop_installed_apps );

            // Get current, next app
            if (
                isset( $_COOKIE[$current_cross_sell_app_key] ) &&
                in_array( $_COOKIE[$current_cross_sell_app_key], $cross_sell_apps )
            ) {
                $current_cross_sell_app = $_COOKIE[$current_cross_sell_app_key];
            } else {
                if (
                    isset( $_COOKIE[$next_cross_sell_app_key] ) &&
                    in_array( $_COOKIE[$next_cross_sell_app_key], $cross_sell_apps )
                ) {
                    $current_cross_sell_app = $_COOKIE[$next_cross_sell_app_key];
                    $next_key = array_search( $current_cross_sell_app, $cross_sell_apps ) + 1;
                    $next_cross_sell_app = isset( $cross_sell_apps[$next_key] ) ? $cross_sell_apps[$next_key] :
                        reset( $cross_sell_apps );
                } else {
                    $current_cross_sell_app = current( $cross_sell_apps );
                    $next_cross_sell_app = next( $cross_sell_apps ) ?: reset( $cross_sell_apps );
                }

                // Set next cross sell app cookie
                setcookie( $current_cross_sell_app_key, $current_cross_sell_app, time() + 86400, '/' );
                setcookie( $next_cross_sell_app_key, $next_cross_sell_app, time() + 604800, '/' );
            }

            return $current_cross_sell_app;
        }
    }

    /**
     * Is app valid for invite upgrade
     *
     * @param $app_code
     * @return bool
     */
    public function is_app_valid_invite_upgrade( $app_code )
    {
        $this->setting_helper->switch_settings( $app_code );
        $upgrade_invited = $this->setting_helper->get_settings( Setting::SETTING_UPGRADE_INVITED );
        $app_api_key = $this->setting_helper->get_settings( Setting::SETTING_API_KEY );
        // App not valid for invited
        if ( $upgrade_invited || !$app_api_key ) {
            return false;
        }

        $data_cookie_key = 'beeketing_apps_data_temp';
        if ( isset( $_COOKIE[$data_cookie_key] ) && $_COOKIE[$data_cookie_key] ) {
            $beeketing_apps_data = unserialize( stripslashes( $_COOKIE[$data_cookie_key] ) );
        } else {
            $beeketing_apps_data = $this->get_apps();

            // Save to cookie
            $new_apps_data = array();
            foreach ( $beeketing_apps_data as $beeketing_app_data ) {
                // Get new data for cookie
                $new_apps_data[] = array(
                    'is_free_trial' => $beeketing_app_data['is_free_trial'],
                    'app_code' => $beeketing_app_data['app_code'],
                    'created_at_timestamp' => $beeketing_app_data['created_at_timestamp'],
                );
            }
            setcookie( $data_cookie_key, serialize( $new_apps_data ), time() + 60, '/' );
        }

        $isShowInviteUpgrade = false;
        foreach ($beeketing_apps_data as $beeketing_app_data) {
            // Get shop installed apps
            if (
                isset(
                    $beeketing_app_data['is_free_trial'],
                    $beeketing_app_data['app_code'],
                    $beeketing_app_data['created_at_timestamp']
                ) &&
                $beeketing_app_data['app_code'] == $app_code &&
                time() - $beeketing_app_data['created_at_timestamp'] > 86400 // 1 day after install
            ) {
                if ( $beeketing_app_data['is_free_trial'] ) {
                    $isShowInviteUpgrade = true;
                } else {
                    $this->setting_helper->update_settings( Setting::SETTING_UPGRADE_INVITED, true );
                }
            }
        }

        if ( !$isShowInviteUpgrade ) {
            return false;
        }

        return true;
    }
}